@@ -31,6 +31,7 @@
 #include "prefs.h"
 #include "file.h"
 #include "dialog-editor.h"
+#include "dialog-tagger.h"
 #include "app-window.h"
 #include "textview.h"
 
@@ -75,6 +76,10 @@ typedef struct {
     GtkTextView         *textview[3];
     GtkTextBuffer       *textbuf[3];
 
+    GtkWidget           *hbox_tag_bar;
+    GtkWidget           *entry_tag_bar;
+    GtkWidget           *b_clear_tag_bar;
+
     GtkWidget           *b_add;
     GtkWidget           *b_remove;
     GtkWidget           *b_prev;
@@ -87,7 +92,8 @@ typedef struct {
     GtkWidget           *m_prev;
     GtkWidget           *m_next;
     GtkCheckMenuItem    *m_flip;
-    GtkCheckMenuItem    *m_tag;
+    GtkCheckMenuItem    *m_tag_bar;
+    GtkCheckMenuItem    *m_flag;
     GtkCheckMenuItem    *m_info;
     GtkWidget           *m_switch_sides;
     GtkWidget           *m_reset_stats;
@@ -97,7 +103,12 @@ typedef struct {
     GtkWidget           *m_insert_image;
     GtkWidget           *m_insert_sound;
 
+    GtkWidget           *m_cut;
+    GtkWidget           *m_copy;
     GtkWidget           *m_paste;
+    GtkWidget           *m_delete;
+
+    gchar               *s_tag_bar;
 
     GList               *undo[2];
     GList               *redo[2];
@@ -110,11 +121,13 @@ typedef struct {
 } Dialog;
 
 
-static void cb_m_tag (GtkCheckMenuItem *widget, Dialog *d);
+static void cb_textbuf_modified (GtkTextBuffer *buf, Dialog *d);
+static void cb_m_flag (GtkCheckMenuItem *widget, Dialog *d);
 static void cb_b_info (GtkToggleButton *tb, Dialog *d);
 static void toggle_ui_info (Dialog *d, gboolean active);
 static void toggle_ui_flip (Dialog *d, gboolean active);
-static void update_title (void);
+static void update_undo_sensitivity (Dialog *d);
+static void update_insert_sensitivity (Dialog *d);
 
 
 static Dialog *dialog = NULL;
@@ -143,9 +156,8 @@ prompt_media_filename (Dialog *d)
     fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(prompt));
     gtk_widget_destroy (prompt);
 
-    if (result == GTK_RESPONSE_ACCEPT) {
+    if (result == GTK_RESPONSE_ACCEPT)
         return fname;
-    }
 
     g_free (fname);
 
@@ -171,11 +183,9 @@ action_new (gint action_type, gint start, gint length, const gchar *str)
 static void
 action_free (Action *a)
 {
-    g_assert (a != NULL);
-
-    if (a->gstr != NULL) {
+    if (a->gstr != NULL)
         g_string_free (a->gstr, TRUE);
-    }
+
     g_free (a);
 }
 
@@ -183,11 +193,11 @@ action_free (Action *a)
 static void
 action_list_free (GList *list)
 {
-    GList *i;
+    GList *item;
+
+    for (item = list; item != NULL; item = item->next)
+        action_free (item->data);
 
-    for (i = list; i != NULL; i = i->next) {
-        action_free (i->data);
-    }
     g_list_free (list);
 }
 
@@ -203,39 +213,6 @@ reset_undo_redo (Dialog *d, gint p)
 }
 
 
-static void
-update_undo_sensitivity (Dialog *d)
-{
-    gboolean can_undo, can_redo;
-    gint p;
-
-    p = gtk_notebook_get_current_page (d->notebook);
-
-    can_undo = (p == FRONT || p == BACK) && (d->undo[p] != NULL);
-    can_redo = (p == FRONT || p == BACK) && (d->redo[p] != NULL);
-
-    gtk_widget_set_sensitive (d->m_undo, can_undo);
-    gtk_widget_set_sensitive (d->m_redo, can_redo);
-}
-
-
-static void
-update_insert_sensitivity (Dialog *d)
-{
-    gboolean can_insert;
-    GList *item;
-    gint p;
-
-    p = gtk_notebook_get_current_page (d->notebook);
-    item = file_get_current_item (d->ig->file);
-
-    can_insert = (item != NULL) && (p == FRONT || p == BACK);
-
-    gtk_widget_set_sensitive (d->m_insert_image, can_insert);
-    gtk_widget_set_sensitive (d->m_insert_sound, can_insert);
-}
-
-
 static void
 cb_textbuf_end_user_action (GtkTextBuffer *textbuf, Dialog *d)
 {
@@ -343,12 +320,10 @@ cb_textbuf_delete_range (GtkTextBuffer *textbuf,
     end = gtk_text_iter_get_offset (i_end);
     length = end - start;
 
-    if (d->keyval == GDK_BackSpace) {
+    if (d->keyval == GDK_BackSpace)
         action = ACTION_BACKSPACE;
-    }
-    else {
+    else
         action = ACTION_DELETE;
-    }
 
     action_add (action, start, length, d);
 }
@@ -357,6 +332,9 @@ cb_textbuf_delete_range (GtkTextBuffer *textbuf,
 static void
 textbuf_block_signals (Dialog *d, gint p)
 {
+    g_signal_handlers_block_by_func (G_OBJECT(d->textbuf[p]),
+        G_CALLBACK(cb_textbuf_modified), d);
+
     g_signal_handlers_block_by_func (G_OBJECT(d->textbuf[p]),
         G_CALLBACK(cb_textbuf_insert_text), d);
     g_signal_handlers_block_by_func (G_OBJECT(d->textbuf[p]),
@@ -367,6 +345,9 @@ textbuf_block_signals (Dialog *d, gint p)
 static void
 textbuf_unblock_signals (Dialog *d, gint p)
 {
+    g_signal_handlers_unblock_by_func (G_OBJECT(d->textbuf[p]),
+        G_CALLBACK(cb_textbuf_modified), d);
+
     g_signal_handlers_unblock_by_func (G_OBJECT(d->textbuf[p]),
         G_CALLBACK(cb_textbuf_insert_text), d);
     g_signal_handlers_unblock_by_func (G_OBJECT(d->textbuf[p]),
@@ -394,11 +375,21 @@ cb_textview_key_press_event (GtkWidget *textview, GdkEventKey *event, Dialog *d)
 }
 
 
-static void
-set_statusbar_message (Dialog *d, const gchar *text)
+static gboolean
+cb_window_key_press_event (GtkWindow *w, GdkEventKey *event, Dialog *d)
 {
-    gtk_statusbar_pop (d->statusbar, d->sbcid);
-    gtk_statusbar_push (d->statusbar, d->sbcid, text);
+    gboolean handled;
+
+    /* This is necessary to prevent copy/paste key presses intended
+     * for the focused tag bar from ending up in the textview.
+     * (This solution found in gedit 2.28.0, gedit-window.c) */
+
+    handled = gtk_window_propagate_key_event (w, event);
+
+    if (!handled)
+        handled = gtk_window_activate_key (w, event);
+
+    return handled;
 }
 
 
@@ -415,7 +406,8 @@ cb_m_undo (GtkWidget *w, Dialog *d)
 
     p = gtk_notebook_get_current_page (d->notebook);
 
-    if (d->undo[p] == NULL) { return; }
+    if (d->undo[p] == NULL)
+        return;
 
     textbuf_block_signals (d, p);
 
@@ -490,7 +482,8 @@ cb_m_redo (GtkWidget *w, Dialog *d)
 
     p = gtk_notebook_get_current_page (d->notebook);
 
-    if (d->redo[p] == NULL) { return; }
+    if (d->redo[p] == NULL)
+        return;
 
     textbuf_block_signals (d, p);
 
@@ -544,7 +537,39 @@ cb_m_redo (GtkWidget *w, Dialog *d)
 
 
 static void
-update_statusbar (Dialog *d)
+update_editor_title (void)
+{
+    GList *item;
+    gchar *title, *flag;
+
+    if (dialog == NULL)
+        return;
+
+    if ((item = file_get_current_item (dialog->ig->file)) != NULL) {
+        flag = card_get_flagged (CARD(item)) ? UNICHAR_FLAGGED : "";
+    }
+    else {
+        flag = "";
+    }
+
+    title = g_strdup_printf (_("Card Editor %s"), flag);
+
+    gtk_window_set_title (GTK_WINDOW(dialog->window), title);
+
+    g_free (title);
+}
+
+
+static void
+set_statusbar_message (Dialog *d, const gchar *text)
+{
+    gtk_statusbar_pop (d->statusbar, d->sbcid);
+    gtk_statusbar_push (d->statusbar, d->sbcid, text);
+}
+
+
+static void
+update_editor_statusbar (Dialog *d)
 {
     GList *item;
     gchar *s;
@@ -582,85 +607,131 @@ update_statusbar (Dialog *d)
 
 
 static void
-set_interface_sensitivity (Dialog *d)
+update_undo_sensitivity (Dialog *d)
+{
+    gboolean can_undo, can_redo;
+    gint p;
+
+    p = gtk_notebook_get_current_page (d->notebook);
+
+    can_undo = (p == FRONT || p == BACK) && (d->undo[p] != NULL);
+    can_redo = (p == FRONT || p == BACK) && (d->redo[p] != NULL);
+
+    gtk_widget_set_sensitive (d->m_undo, can_undo);
+    gtk_widget_set_sensitive (d->m_redo, can_redo);
+}
+
+
+static void
+update_insert_sensitivity (Dialog *d)
+{
+    gboolean can_insert;
+    GList *item;
+    gint p;
+
+    p = gtk_notebook_get_current_page (d->notebook);
+    item = file_get_current_item (d->ig->file);
+
+    can_insert = (item != NULL) && (p == FRONT || p == BACK);
+
+    gtk_widget_set_sensitive (d->m_insert_image, can_insert);
+    gtk_widget_set_sensitive (d->m_insert_sound, can_insert);
+}
+
+
+static void
+update_editor_sensitivity (Dialog *d)
 {
     GdkColor *color;
-    GList    *item;
     Category *cat;
-    gboolean ok;
-    gboolean file_has_cards;
-    gboolean special;
+    gboolean have_card, can_move, can_add;
+
+
+    /* For use when switching between cards. */
 
-    item = file_get_current_item (d->ig->file);
     cat = file_get_current_category (d->ig->file);
-    ok = (cat != NULL && category_get_n_cards (cat) > 1);
-    file_has_cards = (file_get_n_cards (d->ig->file) > 0);
 
-    /* Don't allow addition of cards to special categories:
-     * NULL, search results, trash, or clipboard.
-     */
-    special = (cat == NULL ||
-        file_category_is_search (d->ig->file, cat) ||
-        file_category_is_trash (d->ig->file, cat) ||
-        ig_category_is_clipboard (d->ig, cat));
+    can_add = !(cat == NULL
+        || file_category_is_search (d->ig->file, cat)
+        || file_category_is_trash (d->ig->file, cat)
+        || ig_category_is_clipboard (d->ig, cat));
 
-    gtk_widget_set_sensitive (d->b_prev, ok);
-    gtk_widget_set_sensitive (d->b_next, ok);
-    gtk_widget_set_sensitive (d->m_prev, ok);
-    gtk_widget_set_sensitive (d->m_next, ok);
+    can_move = (cat != NULL && category_get_n_cards (cat) > 1);
 
-    ok = (item != NULL);
+    gtk_widget_set_sensitive (d->b_prev, can_move);
+    gtk_widget_set_sensitive (d->b_next, can_move);
+    gtk_widget_set_sensitive (d->m_prev, can_move);
+    gtk_widget_set_sensitive (d->m_next, can_move);
 
-    gtk_widget_set_sensitive (d->b_add, !special);
-    gtk_widget_set_sensitive (d->b_remove, ok);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->b_flip), ok);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->b_info), ok);
-    gtk_widget_set_sensitive (d->m_add, !special);
-    gtk_widget_set_sensitive (d->m_remove, ok);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->m_flip), ok);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->m_tag), ok);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->m_info), ok);
-    gtk_widget_set_sensitive (d->m_switch_sides, ok);
-    gtk_widget_set_sensitive (d->m_reset_stats, ok);
+    have_card = (file_get_current_item (d->ig->file) != NULL);
+
+    gtk_widget_set_sensitive (d->b_add, can_add);
+    gtk_widget_set_sensitive (d->b_remove, have_card);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->b_flip), have_card);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->b_info), have_card);
+    gtk_widget_set_sensitive (d->m_add, can_add);
+    gtk_widget_set_sensitive (d->m_remove, have_card);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->m_flip), have_card);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->m_flag), have_card);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->m_info), have_card);
+    gtk_widget_set_sensitive (d->m_switch_sides, have_card);
+    gtk_widget_set_sensitive (d->m_reset_stats, have_card);
+
+    gtk_widget_set_sensitive (d->entry_tag_bar, have_card);
+    gtk_widget_set_sensitive (d->b_clear_tag_bar, have_card);
 
     update_undo_sensitivity (d);
     update_insert_sensitivity (d);
 
-    gtk_text_view_set_editable (d->textview[FRONT], ok);
-    gtk_text_view_set_cursor_visible (d->textview[FRONT], ok);
+    gtk_text_view_set_editable (d->textview[FRONT], have_card);
+    gtk_text_view_set_cursor_visible (d->textview[FRONT], have_card);
 
-    color = ok ? prefs_get_color (d->ig->prefs, COLOR_CARD_BG) :
+    color = have_card ? prefs_get_color (d->ig->prefs, COLOR_CARD_BG) :
         prefs_get_color (d->ig->prefs, COLOR_CARD_BG_END);
+
     gtk_widget_modify_base (GTK_WIDGET(d->textview[FRONT]),
         GTK_STATE_NORMAL, color);
+}
 
 
-    /* Main window. */
+static void
+update_main_window_sensitivity (Dialog *d)
+{
+    gboolean have_current, file_not_empty;
 
-    gtk_widget_set_sensitive (d->ig->m_find, file_has_cards);
-    gtk_widget_set_sensitive (d->ig->t_find, file_has_cards);
-    gtk_widget_set_sensitive (d->ig->m_find_all, file_has_cards);
-    gtk_widget_set_sensitive (d->ig->m_select_all, ok);
-    gtk_widget_set_sensitive (d->ig->m_tag, ok);
-    gtk_widget_set_sensitive (d->ig->m_switch_sides, ok);
-    gtk_widget_set_sensitive (d->ig->m_reset_stats, ok);
-    gtk_widget_set_sensitive (d->ig->m_start_quiz, file_has_cards);
-    gtk_widget_set_sensitive (d->ig->t_start_quiz, file_has_cards);
+    /* For use after adding or removing a card. */
 
-    gtk_widget_set_sensitive (d->ig->m_card_popup_select_all, ok);
-    gtk_widget_set_sensitive (d->ig->m_card_popup_tag, ok);
-    gtk_widget_set_sensitive (d->ig->m_card_popup_switch_sides, ok);
-    gtk_widget_set_sensitive (d->ig->m_card_popup_reset_stats, ok);
+    have_current = (file_get_current_item (d->ig->file) != NULL);
+    file_not_empty = (file_get_n_cards (d->ig->file) > 0);
 
+    gtk_widget_set_sensitive (d->ig->m_find, file_not_empty);
+    gtk_widget_set_sensitive (d->ig->t_find, file_not_empty);
+    gtk_widget_set_sensitive (d->ig->m_find_flagged, file_not_empty);
+    gtk_widget_set_sensitive (d->ig->m_find_all, file_not_empty);
+    gtk_widget_set_sensitive (d->ig->m_select_all, have_current);
+    gtk_widget_set_sensitive (d->ig->m_edit_tags, have_current);
+    gtk_widget_set_sensitive (d->ig->m_flag, have_current);
+    gtk_widget_set_sensitive (d->ig->m_switch_sides, have_current);
+    gtk_widget_set_sensitive (d->ig->m_reset_stats, have_current);
+    gtk_widget_set_sensitive (d->ig->m_start_quiz, file_not_empty);
+    gtk_widget_set_sensitive (d->ig->t_start_quiz, file_not_empty);
 
-    update_statusbar (d);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_select_all, have_current);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_edit_tags, have_current);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_flag, have_current);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_switch_sides, have_current);
+    gtk_widget_set_sensitive (d->ig->m_card_popup_reset_stats, have_current);
+
+    gtk_widget_set_sensitive (d->ig->m_view_trash,
+        file_get_trash (d->ig->file) != NULL);
 }
 
 
 static void
 display_text (GtkTextBuffer *textbuf, const gchar *text)
 {
-    if (text == NULL) {text = "";}
+    if (text == NULL)
+        text = "";
 
     textbuf_block_signals (dialog, FRONT);
     textbuf_block_signals (dialog, BACK);
@@ -678,58 +749,87 @@ dialog_blank (void)
 {
     toggle_ui_flip (dialog, FALSE);
     toggle_ui_info (dialog, FALSE);
+
     reset_undo_redo (dialog, FRONT);
     reset_undo_redo (dialog, BACK);
-    display_text (dialog->textbuf[FRONT], NULL);
-    gtk_notebook_set_current_page (dialog->notebook, FRONT);
-    set_interface_sensitivity (dialog);
 
-    update_title ();
+    display_text (dialog->textbuf[FRONT], NULL);
+
+    gtk_notebook_set_current_page (dialog->notebook, FRONT);
+
+    gtk_entry_set_text (GTK_ENTRY(dialog->entry_tag_bar), "");
+
+    update_editor_sensitivity (dialog);
+    update_editor_title ();
 }
 
 
 static void
-show_current_card (Dialog *d)
+show_current_card (Dialog *d, EdTweak tweak)
 {
     GList *item;
-    CardStyle style;
-    Card *c, *c_info;
+    Card *c;
 
 
-    reset_undo_redo (d, FRONT);
-    reset_undo_redo (d, BACK);
-
-    item = file_get_current_item (d->ig->file);
-    g_assert (item != NULL);
-
-    if (item == NULL) {
+    if ((item = file_get_current_item (d->ig->file)) == NULL)
         return;
-    }
 
     c = CARD(item);
 
-    g_assert (c != NULL);
+    if (tweak & ED_TWEAK_TAG_BAR) {
 
-    display_text (d->textbuf[FRONT], card_get_front (c));
-    display_text (d->textbuf[BACK], card_get_back (c));
+        g_free (d->s_tag_bar);
+        d->s_tag_bar = card_get_tags_as_string (c);
+        gtk_entry_set_text (GTK_ENTRY(d->entry_tag_bar), d->s_tag_bar);
 
-    c_info = card_get_details (c);
-    style = card_get_style (c_info);
-    gtk_text_view_set_justification (GTK_TEXT_VIEW(d->textview[INFO]),
-        card_style_get_justification (style));
-    textview_display_with_markup (d->ig, d->textview[INFO],
-        card_get_front (c_info), style, INFO);
-    card_free (c_info);
+    }
 
-    set_interface_sensitivity (d);
+    if (tweak & ED_TWEAK_TEXTVIEW) {
 
-    g_signal_handlers_block_by_func (G_OBJECT(d->m_tag),
-        G_CALLBACK(cb_m_tag), d);
-    gtk_check_menu_item_set_active (d->m_tag, card_get_tagged (c));
-    g_signal_handlers_unblock_by_func (G_OBJECT(d->m_tag),
-        G_CALLBACK(cb_m_tag), d);
+        reset_undo_redo (d, FRONT);
+        reset_undo_redo (d, BACK);
 
-    update_title ();
+        display_text (d->textbuf[FRONT], card_get_front (c));
+        display_text (d->textbuf[BACK], card_get_back (c));
+
+    }
+
+    if (tweak & ED_TWEAK_INFO) {
+
+        CardStyle style;
+        Card *info;
+
+        info = card_get_details (c);
+        style = card_get_style (info);
+
+        gtk_text_view_set_justification (GTK_TEXT_VIEW(d->textview[INFO]),
+            card_style_get_justification (style));
+
+        textview_display_with_markup (d->ig, d->textview[INFO],
+            card_get_front (info), style, INFO);
+
+        card_free (info);
+
+        update_editor_statusbar (d);
+    }
+
+    if (tweak & ED_TWEAK_UI) {
+
+        update_editor_sensitivity (d);
+
+        g_signal_handlers_block_by_func (G_OBJECT(d->m_flag),
+            G_CALLBACK(cb_m_flag), d);
+        gtk_check_menu_item_set_active (d->m_flag, card_get_flagged (c));
+        g_signal_handlers_unblock_by_func (G_OBJECT(d->m_flag),
+            G_CALLBACK(cb_m_flag), d);
+
+    }
+
+    if (tweak & ED_TWEAK_TITLE) {
+
+        update_editor_title ();
+
+    }
 }
 
 
@@ -742,10 +842,9 @@ textview_get_changes (GtkTextView *textview)
 
     textbuf = gtk_text_view_get_buffer (textview);
 
-    /* If no change, return NULL */
-    if (!gtk_text_buffer_get_modified (textbuf)) {
+    /* If no change, return NULL. */
+    if (!gtk_text_buffer_get_modified (textbuf))
         return NULL;
-    }
 
     gtk_text_buffer_get_bounds (textbuf, &start, &end);
     return gtk_text_buffer_get_text (textbuf, &start, &end, FALSE);
@@ -755,19 +854,23 @@ textview_get_changes (GtkTextView *textview)
 static gboolean
 textview_check_changed (Dialog *d)
 {
+    GtkTextBuffer *textbuf;
     GList *item;
     Card *c;
     gboolean changed = FALSE;
     gchar *text;
+    const gchar *s;
 
 
-    item = file_get_current_item (d->ig->file);
-    if (item == NULL) {
+    if ((item = file_get_current_item (d->ig->file)) == NULL)
         return FALSE;
-    }
 
     c = CARD(item);
 
+    s = gtk_entry_get_text (GTK_ENTRY(d->entry_tag_bar));
+    if (strcmp (d->s_tag_bar, s) != 0)
+        changed = file_card_set_tags_from_string (d->ig->file, c, s);
+
     if ((text = textview_get_changes (d->textview[FRONT])) != NULL) {
         changed = TRUE;
         card_set_front (c, text);
@@ -781,11 +884,16 @@ textview_check_changed (Dialog *d)
     }
 
     if (changed) {
-        file_set_changed (d->ig->file, TRUE);
+        ig_file_changed (d->ig);
         app_window_refresh_card_row (d->ig, item);
-        app_window_update_title (d->ig);
     }
 
+    /* In case called from app-window.c cb_m_save, reset modified state: */
+    textbuf = gtk_text_view_get_buffer (d->textview[FRONT]);
+    gtk_text_buffer_set_modified (textbuf, FALSE);
+    textbuf = gtk_text_view_get_buffer (d->textview[BACK]);
+    gtk_text_buffer_set_modified (textbuf, FALSE);
+
     return changed;
 }
 
@@ -806,6 +914,11 @@ window_close (Dialog *d)
     reset_undo_redo (d, FRONT); /* Free undo/redo lists. */
     reset_undo_redo (d, BACK);
 
+    g_free (d->s_tag_bar);
+
+    /* In case tags were removed: */
+    file_delete_unused_tags (d->ig->file);
+
     g_free (d);
     dialog = NULL;
 }
@@ -821,10 +934,7 @@ cb_m_add (GtkWidget *widget, Dialog *d)
 
 
     /* Make a new blank card, add it to the currently selected category,
-     * and set it up for editing.
-     */
-
-    g_assert (file_get_current_category (d->ig->file) != NULL);
+     * and set it up for editing. */
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(d->b_info), FALSE);
 
@@ -836,8 +946,7 @@ cb_m_add (GtkWidget *widget, Dialog *d)
     card_set_front (c, "");
     card_set_back (c, "");
     file_add_card (d->ig->file, cat, c);
-    file_set_changed (d->ig->file, TRUE);
-    app_window_update_title (d->ig);
+    ig_file_changed (d->ig);
 
     item = category_get_cards (cat);
     file_set_current_item (d->ig->file, item);
@@ -845,11 +954,12 @@ cb_m_add (GtkWidget *widget, Dialog *d)
     app_window_add_card_iter (d->ig, &iter, item);
     app_window_refresh_category_row (d->ig, cat);
 
-    show_current_card (d);
+    show_current_card (d, ED_TWEAK_ALL);
 
     gtk_toggle_button_set_active (d->b_flip, FALSE);
 
     app_window_update_appbar (d->ig);
+    update_main_window_sensitivity (d);
 }
 
 
@@ -901,20 +1011,16 @@ cb_m_remove (GtkWidget *widget, Dialog *d)
         file_add_trash (d->ig->file, c);
     }
 
-    if (!file_category_is_trash (d->ig->file, cat_cur)) {
+    if (!file_category_is_trash (d->ig->file, cat_cur))
         app_window_refresh_category_row (d->ig, cat_card);
-    }
 
-    file_set_changed (d->ig->file, TRUE);
-    app_window_update_title (d->ig);
+    ig_file_changed (d->ig);
 
 
     /* Update main window controls and appbar. */
 
-    gtk_widget_set_sensitive (d->ig->m_view_trash,
-        file_get_trash (d->ig->file) != NULL);
-
     app_window_update_appbar (d->ig);
+    update_main_window_sensitivity (d);
 
 
     /* Update the editor window. */
@@ -923,10 +1029,10 @@ cb_m_remove (GtkWidget *widget, Dialog *d)
 
         app_window_select_card (d->ig, next);
 
-        if (!prefs_get_sticky_flips (d->ig->prefs)) {
+        if (!prefs_get_sticky_flips (d->ig->prefs))
             gtk_toggle_button_set_active (d->b_flip, FALSE);
-        }
-        show_current_card (d);
+
+        show_current_card (d, ED_TWEAK_ALL);
         return;
     }
 
@@ -948,22 +1054,15 @@ cb_m_reset_stats (GtkWidget *widget, Dialog *d)
         return;
     }
 
-    textview_check_changed (d);
-
     item = file_get_current_item (d->ig->file);
-    g_assert (item != NULL);
 
     card_reset_statistics (CARD(item));
-    file_set_changed (d->ig->file, TRUE);
-    app_window_update_title (d->ig);
+    ig_file_changed (d->ig);
     app_window_update_appbar (d->ig);
 
     app_window_refresh_card_row (d->ig, item);
 
-    update_statusbar (d);
-
-    /* XXX: The following clears undo/redo lists. */
-    show_current_card (d);
+    show_current_card (d, ED_TWEAK_INFO);
 }
 
 
@@ -972,15 +1071,13 @@ cb_m_prev (GtkWidget *widget, Dialog *d)
 {
     textview_check_changed (d);
 
-    if (!prefs_get_sticky_flips (d->ig->prefs)) {
+    if (!prefs_get_sticky_flips (d->ig->prefs))
         gtk_toggle_button_set_active (d->b_flip, FALSE);
-    }
 
     file_decr_current_item (d->ig->file, TRUE);
-    app_window_select_card (d->ig,
-        file_get_current_item (d->ig->file));
+    app_window_select_card (d->ig, file_get_current_item (d->ig->file));
 
-    show_current_card (d);
+    show_current_card (d, ED_TWEAK_ALL);
 }
 
 
@@ -989,15 +1086,13 @@ cb_m_next (GtkWidget *widget, Dialog *d)
 {
     textview_check_changed (d);
 
-    if (!prefs_get_sticky_flips (d->ig->prefs)) {
+    if (!prefs_get_sticky_flips (d->ig->prefs))
         gtk_toggle_button_set_active (d->b_flip, FALSE);
-    }
 
     file_incr_current_item (d->ig->file, TRUE);
-    app_window_select_card (d->ig,
-        file_get_current_item (d->ig->file));
+    app_window_select_card (d->ig, file_get_current_item (d->ig->file));
 
-    show_current_card (d);
+    show_current_card (d, ED_TWEAK_ALL);
 }
 
 
@@ -1012,9 +1107,8 @@ cb_m_flip (GtkCheckMenuItem *widget, Dialog *d)
     active = gtk_check_menu_item_get_active (widget);
 
     toggle_ui_flip (d, active);
-    if (gtk_toggle_button_get_active (d->b_info) == FALSE) {
+    if (gtk_toggle_button_get_active (d->b_info) == FALSE)
         gtk_notebook_set_current_page (d->notebook, active ? BACK : FRONT);
-    }
 
     update_undo_sensitivity (d);
 }
@@ -1046,9 +1140,8 @@ cb_m_switch_sides (GtkWidget *widget, Dialog *d)
     textview_check_changed (d);
 
     card_switch_sides (file_get_current_card (d->ig->file));
-    file_set_changed (d->ig->file, TRUE);
-    app_window_update_title (d->ig);
-    show_current_card (d);
+    ig_file_changed (d->ig);
+    show_current_card (d, ED_TWEAK_TEXTVIEW);
 
     /* Update this item in the main window's card list. */
     item = file_get_current_item (d->ig->file);
@@ -1057,7 +1150,7 @@ cb_m_switch_sides (GtkWidget *widget, Dialog *d)
 
 
 static void
-cb_m_tag (GtkCheckMenuItem *widget, Dialog *d)
+cb_m_flag (GtkCheckMenuItem *widget, Dialog *d)
 {
     gboolean state;
     GList *item;
@@ -1065,20 +1158,18 @@ cb_m_tag (GtkCheckMenuItem *widget, Dialog *d)
 
 
     item = file_get_current_item (d->ig->file);
-    g_assert (item != NULL);
 
     c = CARD(item);
 
     state = gtk_check_menu_item_get_active (widget);
 
-    if (card_get_tagged (c) != state) {
-        card_set_tagged (c, state);
-        file_set_changed (d->ig->file, TRUE);
+    if (card_get_flagged (c) != state) {
+        card_set_flagged (c, state);
+        ig_file_changed (d->ig);
         app_window_refresh_card_row (d->ig, item);
-        app_window_update_title (d->ig);
     }
 
-    update_title ();
+    update_editor_title ();
 }
 
 
@@ -1095,7 +1186,8 @@ cb_m_cut (GtkWidget *w, Dialog *d)
     gint p;
 
     p = gtk_notebook_get_current_page (d->notebook);
-    if (p != FRONT && p != BACK) { return; }
+    if (p != FRONT && p != BACK)
+        return;
 
     gtk_text_buffer_cut_clipboard (d->textbuf[p],
         gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), TRUE);
@@ -1119,10 +1211,12 @@ cb_m_paste (GtkWidget *w, Dialog *d)
 {
     gint p;
 
-    if (file_get_current_item (d->ig->file) == NULL) { return; }
+    if (file_get_current_item (d->ig->file) == NULL)
+        return;
 
     p = gtk_notebook_get_current_page (d->notebook);
-    if (p != FRONT && p != BACK) { return; }
+    if (p != FRONT && p != BACK)
+        return;
 
     gtk_text_buffer_paste_clipboard (d->textbuf[p],
         gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), NULL, TRUE);
@@ -1137,7 +1231,8 @@ cb_m_delete (GtkWidget *w, Dialog *d)
     gint p;
 
     p = gtk_notebook_get_current_page (d->notebook);
-    if (p != FRONT && p != BACK) { return; }
+    if (p != FRONT && p != BACK)
+        return;
 
     gtk_text_buffer_delete_selection (d->textbuf[FRONT], TRUE, TRUE);
 }
@@ -1155,7 +1250,8 @@ cb_m_insert_media (GtkWidget *w, Dialog *d)
         gint p;
 
         p = gtk_notebook_get_current_page (d->notebook);
-        if (p != FRONT && p != BACK) { return; }
+        if (p != FRONT && p != BACK)
+            return;
 
         if (w == d->m_insert_image) {
             type = "image";
@@ -1169,8 +1265,8 @@ cb_m_insert_media (GtkWidget *w, Dialog *d)
 
         /* XXX: What if fname contains a " char? */
         s = g_strdup_printf ("<embed type=\"%s\" src=\"%s\" alt=\"[%s]\"/>", type, fname, type);
-        gtk_text_buffer_insert_interactive_at_cursor
-            (d->textbuf[p], s, strlen (s), TRUE);
+        gtk_text_buffer_insert_interactive_at_cursor (d->textbuf[p],
+            s, strlen (s), TRUE);
 
         g_free (fname);
         g_free (s);
@@ -1251,6 +1347,67 @@ toggle_ui_flip (Dialog *d, gboolean active)
 }
 
 
+static void
+cb_m_tag_bar (GtkCheckMenuItem *widget, Dialog *d)
+{
+    gboolean active;
+
+    active =  gtk_check_menu_item_get_active (widget);
+    if (active) {
+        gtk_widget_show (d->hbox_tag_bar);
+        gtk_widget_grab_focus (d->entry_tag_bar);
+    }
+    else {
+        gtk_widget_hide (d->hbox_tag_bar);
+    }
+
+    prefs_set_editor_tag_bar_visible (d->ig->prefs, active);
+}
+
+
+static void
+cb_b_close_tag_bar (GtkWidget *widget, Dialog *d)
+{
+    gtk_check_menu_item_set_active (d->m_tag_bar, FALSE);
+    gtk_widget_hide (d->hbox_tag_bar);
+
+    prefs_set_editor_tag_bar_visible (d->ig->prefs, FALSE);
+}
+
+
+static void
+cb_tag_bar_activate (GtkWidget *widget, Dialog *d)
+{
+    GList *item;
+    const gchar *s;
+    gboolean changed;
+    Card *c;
+
+
+    item = file_get_current_item (d->ig->file);
+
+    c = CARD(item);
+
+    s = gtk_entry_get_text (GTK_ENTRY(d->entry_tag_bar));
+    if (strcmp (d->s_tag_bar, s) != 0)
+        changed = file_card_set_tags_from_string (d->ig->file, c, s);
+
+    if (changed) {
+        ig_file_changed (d->ig);
+        /* app_window_refresh_card_row (d->ig, item); */
+        dialog_tagger_tweak ();
+    }
+}
+
+
+static void
+cb_b_clear_tag_bar (GtkWidget *widget, Dialog *d)
+{
+    gtk_entry_set_text (GTK_ENTRY(d->entry_tag_bar), "");
+    cb_tag_bar_activate (widget, d);
+}
+
+
 static void
 cb_window_delete (GtkWidget *widget, GdkEvent *event, Dialog *d)
 {
@@ -1258,83 +1415,85 @@ cb_window_delete (GtkWidget *widget, GdkEvent *event, Dialog *d)
 }
 
 
+static void
+cb_textbuf_modified (GtkTextBuffer *buf, Dialog *d)
+{
+    ig_file_changed (d->ig);
+}
+
+
 void
 dialog_editor_check_changed (void)
 {
-    if (dialog) {
+    if (dialog != NULL)
         textview_check_changed (dialog);
-    }
 }
 
 
 void
 dialog_editor_kill (void)
 {
-    if (dialog) {
+    if (dialog != NULL)
         window_close (dialog);
-    }
 }
 
 
 void
-dialog_editor_tweak (void)
+dialog_editor_tweak (EdTweak tweak)
 {
-    if (!dialog) { return; }
+    if (dialog == NULL)
+        return;
 
     if (!file_get_current_item (dialog->ig->file)) {
         dialog_blank ();
         return;
     }
 
-    show_current_card (dialog);
+    show_current_card (dialog, tweak);
+}
+
+
+static void
+update_textview_properties (Dialog *d)
+{
+    textview_set_properties (d->textview[FRONT], d->ig->prefs,
+        file_get_card_style (d->ig->file), TV_EDITOR);
+    textview_set_properties (d->textview[BACK], d->ig->prefs,
+        file_get_card_style (d->ig->file), TV_EDITOR);
+    textview_set_properties (d->textview[INFO], d->ig->prefs,
+        file_get_card_style (d->ig->file), TV_INFO);
 }
 
 
 void
 dialog_editor_preferences_changed (void)
 {
-    if (!dialog) { return; }
+    if (dialog == NULL)
+        return;
 
     if (!file_get_current_item (dialog->ig->file)) {
         dialog_blank ();
         return;
     }
 
-    textview_set_properties (dialog->textview[FRONT], dialog->ig->prefs,
-        file_get_card_style (dialog->ig->file), TV_EDITOR);
-    textview_set_properties (dialog->textview[BACK], dialog->ig->prefs,
-        file_get_card_style (dialog->ig->file), TV_EDITOR);
-    textview_set_properties (dialog->textview[INFO], dialog->ig->prefs,
-        file_get_card_style (dialog->ig->file), TV_INFO);
+    update_textview_properties (dialog);
 }
 
 
-static void
-update_title (void)
+void
+dialog_editor_card_style_changed (void)
 {
-    GList *item;
-    Card *c;
-    gchar *title, *flag = "";
+    CardStyle style;
 
-    if (!dialog) { return; }
+    if (dialog == NULL)
+        return;
 
-    item = file_get_current_item (dialog->ig->file);
+    style = file_get_card_style (dialog->ig->file);
 
-    if (item != NULL) {
-
-        c = CARD(item);
-        g_assert (c != NULL);
-
-        if (card_get_tagged (c)) {
-          flag = UNICHAR_TAGGED;
-        }
-    }
-
-    title = g_strdup_printf (_("Card Editor %s"), flag);
-
-    gtk_window_set_title (GTK_WINDOW(dialog->window), title);
-
-    g_free (title);
+    gtk_text_view_set_justification (GTK_TEXT_VIEW(dialog->textview[FRONT]),
+        card_style_get_justification (style));
+    gtk_text_view_set_justification (GTK_TEXT_VIEW(dialog->textview[BACK]),
+        card_style_get_justification (style));
 }
 
 
@@ -1342,8 +1501,7 @@ void
 dialog_editor (Ignuit *ig)
 {
     Dialog     *d;
-    GtkWidget  *m_close;
-    GtkWidget  *m_cut, *m_copy, *m_delete;
+    GtkWidget  *m_close, *b_close_tag_bar;
     GtkWidget  *m_sticky_flips;
     gchar      *glade_file;
     GladeXML   *glade_xml;
@@ -1351,7 +1509,7 @@ dialog_editor (Ignuit *ig)
 
     if (dialog != NULL) {
         gtk_window_present (GTK_WINDOW(dialog->window));
-        show_current_card (dialog);
+        show_current_card (dialog, ED_TWEAK_ALL);
         return;
     }
 
@@ -1372,6 +1530,9 @@ dialog_editor (Ignuit *ig)
 
     d->window = glade_xml_get_widget (glade_xml, "dialog");
 
+    g_signal_connect (G_OBJECT(d->window), "key-press-event",
+        G_CALLBACK(cb_window_key_press_event), d);
+
     gtk_window_set_transient_for (GTK_WINDOW(d->window),
         GTK_WINDOW(ig->app));
     gtk_window_set_modal (GTK_WINDOW(d->window), FALSE);
@@ -1406,6 +1567,8 @@ dialog_editor (Ignuit *ig)
         G_CALLBACK(cb_textbuf_delete_range), d);
     g_signal_connect_after (G_OBJECT(d->textbuf[FRONT]), "end-user-action",
         G_CALLBACK(cb_textbuf_end_user_action), d);
+    g_signal_connect_after (G_OBJECT(d->textbuf[FRONT]), "modified_changed",
+        G_CALLBACK(cb_textbuf_modified), d);
 
 
     /* Back textview */
@@ -1424,6 +1587,8 @@ dialog_editor (Ignuit *ig)
         G_CALLBACK(cb_textbuf_delete_range), d);
     g_signal_connect_after (G_OBJECT(d->textbuf[BACK]), "end-user-action",
         G_CALLBACK(cb_textbuf_end_user_action), d);
+    g_signal_connect_after (G_OBJECT(d->textbuf[BACK]), "modified_changed",
+        G_CALLBACK(cb_textbuf_modified), d);
 
 
     /* Details textview */
@@ -1469,6 +1634,14 @@ dialog_editor (Ignuit *ig)
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(m_sticky_flips),
         prefs_get_sticky_flips (d->ig->prefs));
 
+    d->m_tag_bar = (GtkCheckMenuItem*)glade_xml_get_widget (glade_xml,
+        "m_tag_bar");
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(d->m_tag_bar),
+        prefs_get_editor_tag_bar_visible (d->ig->prefs));
+
+    g_signal_connect (G_OBJECT(d->m_tag_bar), "toggled",
+        G_CALLBACK(cb_m_tag_bar), d);
+
     m_close = glade_xml_get_widget (glade_xml, "m_close");
     g_signal_connect (G_OBJECT(m_close), "activate",
         G_CALLBACK(cb_m_close), d);
@@ -1484,16 +1657,16 @@ dialog_editor (Ignuit *ig)
     g_signal_connect (G_OBJECT(d->m_redo), "activate",
         G_CALLBACK(cb_m_redo), d);
 
-    m_cut = glade_xml_get_widget (glade_xml, "m_cut");
-    g_signal_connect (G_OBJECT(m_cut), "activate",
+    d->m_cut = glade_xml_get_widget (glade_xml, "m_cut");
+    g_signal_connect (G_OBJECT(d->m_cut), "activate",
         G_CALLBACK(cb_m_cut), d);
 
-    m_copy = glade_xml_get_widget (glade_xml, "m_copy");
-    g_signal_connect (G_OBJECT(m_copy), "activate",
+    d->m_copy = glade_xml_get_widget (glade_xml, "m_copy");
+    g_signal_connect (G_OBJECT(d->m_copy), "activate",
         G_CALLBACK(cb_m_copy), d);
 
-    m_delete = glade_xml_get_widget (glade_xml, "m_delete");
-    g_signal_connect (G_OBJECT(m_delete), "activate",
+    d->m_delete = glade_xml_get_widget (glade_xml, "m_delete");
+    g_signal_connect (G_OBJECT(d->m_delete), "activate",
         G_CALLBACK(cb_m_delete), d);
 
     d->m_paste = glade_xml_get_widget (glade_xml, "m_paste");
@@ -1508,9 +1681,9 @@ dialog_editor (Ignuit *ig)
     g_signal_connect (G_OBJECT(d->m_insert_sound), "activate",
         G_CALLBACK(cb_m_insert_media), d);
 
-    d->m_tag = (GtkCheckMenuItem*)glade_xml_get_widget (glade_xml, "m_tag");
-    g_signal_connect (G_OBJECT(d->m_tag), "toggled",
-        G_CALLBACK(cb_m_tag), d);
+    d->m_flag = (GtkCheckMenuItem*)glade_xml_get_widget (glade_xml, "m_flag");
+    g_signal_connect (G_OBJECT(d->m_flag), "toggled",
+        G_CALLBACK(cb_m_flag), d);
 
     d->m_switch_sides = glade_xml_get_widget (glade_xml, "m_switch_sides");
     g_signal_connect (G_OBJECT(d->m_switch_sides), "activate",
@@ -1521,6 +1694,24 @@ dialog_editor (Ignuit *ig)
         G_CALLBACK(cb_m_reset_stats), d);
 
 
+    /* Tag bar */
+
+    d->hbox_tag_bar = glade_xml_get_widget (glade_xml, "hbox_tag_bar");
+    d->entry_tag_bar = glade_xml_get_widget (glade_xml, "entry_tag_bar");
+    d->b_clear_tag_bar = glade_xml_get_widget (glade_xml, "b_clear_tag_bar");
+    b_close_tag_bar = glade_xml_get_widget (glade_xml, "b_close_tag_bar");
+
+    g_signal_connect (G_OBJECT(b_close_tag_bar), "clicked",
+        G_CALLBACK(cb_b_close_tag_bar), d);
+    g_signal_connect (G_OBJECT(d->b_clear_tag_bar), "clicked",
+        G_CALLBACK(cb_b_clear_tag_bar), d);
+    g_signal_connect (G_OBJECT(d->entry_tag_bar), "activate",
+        G_CALLBACK(cb_tag_bar_activate), d);
+
+    if (!prefs_get_editor_tag_bar_visible (d->ig->prefs))
+        gtk_widget_hide (d->hbox_tag_bar);
+
+
     /* Navigation and scoring buttons */
 
     d->b_info = (GtkToggleButton*)glade_xml_get_widget (glade_xml, "b_info");
@@ -1547,7 +1738,7 @@ dialog_editor (Ignuit *ig)
     g_signal_connect (G_OBJECT(d->b_flip), "toggled",
         G_CALLBACK(cb_b_flip), d);
 
-    show_current_card (d);
+    show_current_card (d, ED_TWEAK_ALL);
 
     gtk_widget_show (d->window);
 
