@@ -38,13 +38,11 @@
 #include "card.h"
 #include "file.h"
 #include "fileio.h"
+#include "csvio.h"
 #include "app-window.h"
 
 
-/* Place-holders for importing/exporting CSV and TSV files: */
-
-#define ESCAPE_NEWLINE_S  "&#10;"
-#define ESCAPE_COMMA_S    "&#44;"
+#define ENABLE_IGNUIT_DEPRECATED 1
 
 
 typedef enum {
@@ -68,16 +66,16 @@ struct _Parser {
     gint        unknown_depth;
     gboolean    dodgy_file;
     File        *file;
-    Card        *card;        /* currently loading card */
+    Card        *card;        /* The currently loading card. */
 };
 
 
 static void
 parser_start_unknown (Parser *parser)
 {
-    if (parser->unknown_depth == 0) {
+    if (parser->unknown_depth == 0)
         parser->state_last_known = parser->state;
-    }
+
     parser->state = STATE_UNKNOWN;
     parser->unknown_depth++;
     parser->dodgy_file = TRUE;
@@ -88,9 +86,10 @@ static void
 parser_end_unknown (Parser *parser)
 {
     parser->unknown_depth--;
-    if (parser->unknown_depth == 0) {
+
+    if (parser->unknown_depth == 0)
         parser->state = parser->state_last_known;
-    }
+
     parser->dodgy_file = TRUE;
 }
 
@@ -99,9 +98,10 @@ static const gchar*
 attr_value (const gchar *name, const gchar **attrn, const gchar **attrv)
 {
     while (*attrn && *attrv) {
-        if (strcmp (*attrn, name) == 0) {
+
+        if (strcmp (*attrn, name) == 0)
              return *attrv;
-        }
+
         attrn++;
         attrv++;
     }
@@ -122,98 +122,165 @@ start_element_handler (GMarkupParseContext *context,
     parser->depth = parser->depth + 1;
 
     switch (parser->state) {
+
     case STATE_START:
+
         if (strcmp (element_name, "deck") == 0) {
+
             parser->state = STATE_IN_DECK;
+
             s = attr_value ("title", attribute_names, attribute_values);
             file_set_title (parser->file, s);
+
             s = attr_value ("author", attribute_names, attribute_values);
             file_set_author (parser->file, s);
+
+#if ENABLE_IGNUIT_DEPRECATED
+
+            /* "comment" and "url" were replaced with "description"
+             * and "homepage" in ignuit 0.0.16 */
+
             s = attr_value ("comment", attribute_names, attribute_values);
-            file_set_comment (parser->file, s);
+            file_set_description (parser->file, s);
+
             s = attr_value ("url", attribute_names, attribute_values);
-            file_set_url (parser->file, s);
+            file_set_homepage (parser->file, s);
+
+            s = attr_value ("description", attribute_names, attribute_values);
+            if (s != NULL)
+                file_set_description (parser->file, s);
+
+            s = attr_value ("homepage", attribute_names, attribute_values);
+            if (s != NULL)
+                file_set_homepage (parser->file, s);
+#else
+            s = attr_value ("description", attribute_names, attribute_values);
+            file_set_description (parser->file, s);
+
+            s = attr_value ("homepage", attribute_names, attribute_values);
+            file_set_homepage (parser->file, s);
+#endif
             s = attr_value ("license", attribute_names, attribute_values);
             file_set_license (parser->file, s);
+
+            s = attr_value ("license-uri", attribute_names, attribute_values);
+            file_set_license_uri (parser->file, s);
+
             s = attr_value ("style", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 file_set_card_style (parser->file, atoi (s));
-            }
+
         }
         else {
+
             parser_start_unknown (parser);
+
         }
         break;
+
     case STATE_IN_DECK:
+
         if (strcmp (element_name, "category") == 0) {
+
             Category *cat;
+
             parser->state = STATE_IN_CATEGORY;
             s = attr_value ("title", attribute_names, attribute_values);
             cat = category_new (s);
             file_add_category (parser->file, cat);
             file_set_current_category (parser->file, cat);
+
         }
         else {
+
             parser_start_unknown (parser);
+
         }
         break;
+
     case STATE_IN_CATEGORY:
+
         if (strcmp (element_name, "card") == 0) {
+
             parser->state = STATE_IN_CARD;
             parser->card = card_new ();
+
             s = attr_value ("grp", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_group (parser->card, atoi (s));
-            }
+
             s = attr_value ("crt", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_date_created (parser->card, strtoul (s, NULL, 10));
-            }
+
             s = attr_value ("tst", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_date_tested (parser->card, strtoul (s, NULL, 10));
-            }
+
             s = attr_value ("exp", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_date_expiry (parser->card, strtoul (s, NULL, 10));
-            }
+
             s = attr_value ("time", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_time_expiry (parser->card, atoi (s));
-            }
+
             s = attr_value ("n", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_n_tests (parser->card, atoi (s));
-            }
+
             s = attr_value ("c", attribute_names, attribute_values);
-            if (s != NULL) {
+            if (s != NULL)
                 card_set_n_known (parser->card, atoi (s));
-            }
+
+#if ENABLE_IGNUIT_DEPRECATED
+
+            /* "t" (for tag) was replaced with "f" (for flag)
+             * in ignuit 0.0.16 */
+
             s = attr_value ("t", attribute_names, attribute_values);
-            if (s != NULL) {
-                card_set_tagged (parser->card, atoi (s) != 0);
-            }
+            if (s != NULL)
+                card_set_flagged (parser->card, atoi (s) != 0);
+
+#endif
+            s = attr_value ("f", attribute_names, attribute_values);
+            if (s != NULL)
+                card_set_flagged (parser->card, atoi (s) != 0);
+
+            s = attr_value ("tags", attribute_names, attribute_values);
+            if (s != NULL)
+                file_card_add_new_tags (parser->file,
+                    parser->card, s);
+
         }
         else {
+
             parser_start_unknown (parser);
+
         }
         break;
+
     case STATE_IN_CARD:
-        if (strcmp (element_name, "front") == 0) {
+
+        if (strcmp (element_name, "front") == 0)
             parser->state = STATE_IN_FRONT;
-        }
-        else if (strcmp (element_name, "back") == 0) {
+
+        else if (strcmp (element_name, "back") == 0)
             parser->state = STATE_IN_BACK;
-        }
-        else {
+
+        else
             parser_start_unknown (parser);
-        }
+
         break;
+
     case STATE_IN_FRONT:
     case STATE_IN_BACK:
     case STATE_UNKNOWN:
+
         parser_start_unknown (parser);
+
         break;
+
     }
 }
 
@@ -229,19 +296,19 @@ end_element_handler (GMarkupParseContext *context,
     parser->depth = parser->depth - 1;
 
     switch (parser->state) {
+
     case STATE_IN_DECK:
-        if (strcmp (element_name, "deck") != 0) {
+        if (strcmp (element_name, "deck") != 0)
             parser_end_unknown (parser);
-        }
         break;
+
     case STATE_IN_CATEGORY:
-        if (strcmp (element_name, "category") == 0) {
+        if (strcmp (element_name, "category") == 0)
             parser->state = STATE_IN_DECK;
-        }
-        else {
+        else
             parser_end_unknown (parser);
-        }
         break;
+
     case STATE_IN_CARD:
         if (strcmp (element_name, "card") == 0) {
             parser->state = STATE_IN_CATEGORY;
@@ -252,28 +319,28 @@ end_element_handler (GMarkupParseContext *context,
             parser_end_unknown (parser);
         }
         break;
+
     case STATE_IN_FRONT:
-        if (strcmp (element_name, "front") == 0) {
+        if (strcmp (element_name, "front") == 0)
             parser->state = STATE_IN_CARD;
-        }
-        else {
+        else
             parser_end_unknown (parser);
-        }
         break;
+
     case STATE_IN_BACK:
-        if (strcmp (element_name, "back") == 0) {
+        if (strcmp (element_name, "back") == 0)
             parser->state = STATE_IN_CARD;
-        }
-        else {
+        else
             parser_end_unknown (parser);
-        }
         break;
+
     case STATE_UNKNOWN:
         parser_end_unknown (parser);
         break;
+
     case STATE_START:
-        g_assert_not_reached ();
         break;
+
     }
 }
 
@@ -321,9 +388,8 @@ fileio_load (const gchar *fname, GError **err)
 
     /* Load a native ignuit XML file. */
 
-    if (!g_file_get_contents (fname, &contents, &len, err)) {
+    if (!g_file_get_contents (fname, &contents, &len, err))
         return NULL;
-    }
 
     if (!g_utf8_validate (contents, -1, NULL)) {
         g_set_error (err, 0, 0, _("The file is not valid UTF-8."));
@@ -364,14 +430,13 @@ fileio_load (const gchar *fname, GError **err)
 
     /* Don't use the GError set by g_markup_parse_context_parse, because
      * it likely contains markup that we don't want to feed into an error
-     * dialog.
-     */
+     * dialog. */
 
     g_set_error (err, 0, 0, _("Error while parsing '%s'"), fname);
 
-    if (parser.card != NULL) {
+    if (parser.card != NULL)
         card_free (parser.card);
-    }
+
     file_free (parser.file, TRUE);
 
     return NULL;
@@ -389,28 +454,39 @@ write_card_xml (FILE *fp, Card *c)
 
     fprintf (fp, "<card grp='%d'", card_get_group (c));
 
-    if ((date = card_get_date_created (c)) != G_DATE_BAD_JULIAN) {
+    if ((date = card_get_date_created (c)) != G_DATE_BAD_JULIAN)
         fprintf (fp, " crt='%d'", date);
-    }
 
-    if ((date = card_get_date_tested (c)) != G_DATE_BAD_JULIAN) {
+    if ((date = card_get_date_tested (c)) != G_DATE_BAD_JULIAN)
         fprintf (fp, " tst='%d'", date);
-    }
 
     if ((date = card_get_date_expiry (c)) != G_DATE_BAD_JULIAN) {
         fprintf (fp, " exp='%d'", date);
-        if ((hour = card_get_time_expiry (c)) != 0) {
+        if ((hour = card_get_time_expiry (c)) != 0)
             fprintf (fp, " time='%d'", hour);
-        }
     }
 
     n = card_get_n_tests (c);
-    if (n > 0) { fprintf (fp, " n='%d'", n); }
+    if (n > 0)
+        fprintf (fp, " n='%d'", n);
 
     n = card_get_n_known (c);
-    if (n > 0) { fprintf (fp, " c='%d'", n); }
+    if (n > 0)
+        fprintf (fp, " c='%d'", n);
 
-    if (card_get_tagged (c)) { fprintf (fp, " t='1'"); }
+    if (card_get_flagged (c))
+        fprintf (fp, " f='1'");
+
+    if (card_get_tags (c) != NULL) {
+
+        gchar *tags;
+
+        tags = card_get_tags_as_string (c);
+        s = g_markup_printf_escaped ("%s", tags);
+        fprintf (fp, " tags='%s'", s);
+        g_free (s);
+        g_free (tags);
+    }
 
     text = card_get_front (c);
     s = g_markup_printf_escaped ("><front>%s</front>", text);
@@ -435,9 +511,8 @@ write_category_xml (FILE *fp, Category *cat)
     fprintf (fp, "<category title='%s'>\n", s);
     g_free (s);
 
-    for (cur = category_get_cards (cat); cur != NULL; cur = cur->next) {
+    for (cur = category_get_cards (cat); cur != NULL; cur = cur->next)
         write_card_xml (fp, CARD(cur));
-    }
 
     fprintf (fp, "</category>\n");
 }
@@ -464,30 +539,61 @@ write_file_xml (Ignuit *ig, FILE *fp)
     fprintf (fp, " author='%s'", s);
     g_free (s);
 
-    s = g_markup_escape_text (file_get_comment (ig->file), -1);
-    fprintf (fp, " comment='%s'", s);
+    s = g_markup_escape_text (file_get_description (ig->file), -1);
+    fprintf (fp, " description='%s'", s);
     g_free (s);
 
-    s = g_markup_escape_text (file_get_url (ig->file), -1);
-    fprintf (fp, " url='%s'", s);
+    s = g_markup_escape_text (file_get_homepage (ig->file), -1);
+    fprintf (fp, " homepage='%s'", s);
     g_free (s);
 
     s = g_markup_escape_text (file_get_license (ig->file), -1);
     fprintf (fp, " license='%s'", s);
     g_free (s);
 
+    s = g_markup_escape_text (file_get_license_uri (ig->file), -1);
+    fprintf (fp, " license-uri='%s'", s);
+    g_free (s);
+
     fprintf (fp, " style='%d'", file_get_card_style (ig->file));
 
     fprintf (fp, ">\n");
 
-    for (cur = file_get_category_order (ig->file); cur != NULL; cur = cur->next) {
+    for (cur = file_get_category_order (ig->file); cur != NULL; cur = cur->next)
         write_category_xml (fp, CATEGORY(cur));
-    }
 
     fprintf (fp, "</deck>\n");
 }
 
 
+static gboolean
+make_backup (const gchar *fname, GError **err)
+{
+    gchar *fname_backup;
+    gboolean ok;
+
+    if (!g_file_test (fname, G_FILE_TEST_EXISTS))
+        return TRUE;
+
+    if (!g_file_test (fname, G_FILE_TEST_IS_REGULAR)) {
+        g_set_error (err, 0, 0, "Not a regular file: '%s'", fname);
+        return FALSE;
+    }
+
+    fname_backup = g_strdup_printf ("%s~", fname);
+
+    ok = (g_rename (fname, fname_backup) == 0);
+    if (!ok) {
+        g_set_error (err, 0, 0, "Failed to rename '%s' as '%s'",
+            fname, fname_backup);
+    }
+
+    g_free (fname_backup);
+
+    return ok;
+}
+
+
 gboolean
 fileio_save (Ignuit *ig, const gchar *fname, GError **err)
 {
@@ -496,7 +602,8 @@ fileio_save (Ignuit *ig, const gchar *fname, GError **err)
 
     /* Write a native ignuit XML file. */
 
-    g_assert (fname != NULL);
+    if (prefs_get_backup (ig->prefs) && !make_backup (fname, err))
+        return FALSE;
 
     f_strm = fopen (fname, "w");
     if (!f_strm) {
@@ -514,85 +621,89 @@ fileio_save (Ignuit *ig, const gchar *fname, GError **err)
 }
 
 
-static gchar*
-escape_with (gchar *text, const gchar *find, const gchar *replace)
+static void
+load_card_from_csv_row (File *f, GList *row)
 {
-    gchar **v, *new_text;
+    GList *item;
+    Card *c;
 
-    v = g_strsplit (text, find, -1);
-    new_text = g_strjoinv (replace, v);
+    c = card_new ();
 
-    g_strfreev (v);
-    g_free (text);
+    if ((item = g_list_nth (row, 0)) != NULL)
+        card_set_front (c, (gchar*)item->data);
 
-    return new_text;
+    if ((item = g_list_nth (row, 1)) != NULL)
+        card_set_back (c, (gchar*)item->data);
+
+    if ((item = g_list_nth (row, 2)) != NULL) {
+
+        Category *cat;
+        gchar *title;
+
+        title = (gchar*)item->data;
+        cat = file_lookup_category (f, title);
+
+        if (cat == NULL) {
+            cat = category_new (title);
+            file_add_category (f, cat);
+        }
+        file_set_current_category (f, cat);  
+    }
+
+    if (file_get_current_category (f) == NULL) {
+        Category *cat = category_new (NULL);
+        file_add_category (f, cat);
+        file_set_current_category (f, cat);
+    }
+
+    file_add_loaded_card (f, c);
 }
 
 
 File*
-fileio_import_sv (const gchar *fname, const gchar *sep, GError **err)
+fileio_import_csv (const gchar *fname, gchar delimiter, GError **err)
 {
-    gchar    *contents;
+    GError   *parse_err = NULL;
+    Csv      *csv;
     File     *f;
-    Category *cat;
-    gchar    **lines, **s;
+    GList    *row;
+    gchar    *contents;
 
 
     /* Import a file from Comma or Tab Separated Values. */
 
-    if (!g_file_get_contents (fname, &contents, NULL, err)) {
+    /* First check for valid UTF-8. */
+
+    if (!g_file_get_contents (fname, &contents, NULL, err))
         return NULL;
-    }
 
     if (!g_utf8_validate (contents, -1, NULL)) {
         g_set_error (err, 0, 0, _("The file is not valid UTF-8."));
         g_free (contents);
         return NULL;
     }
-
-    f = file_new ();
-    cat = category_new (NULL);
-    file_add_category (f, cat);
-    file_set_current_category (f, cat);
-
-    lines = g_strsplit (contents, "\n", -1);
-
-    for (s = lines; *s != NULL; s++) {
-
-        gchar **pair;
-
-        pair = g_strsplit (*s, sep, 2);
-
-        if (pair[0] && pair[1]) {
-
-            gchar *front, *back;
-            Card *c;
-
-            front = g_strstrip (g_strdup (pair[0]));
-            front = escape_with (front, ESCAPE_NEWLINE_S, "\n");
-            front = escape_with (front, ESCAPE_COMMA_S, ",");
-
-            back = g_strstrip (g_strdup (pair[1]));
-            back = escape_with (back, ESCAPE_NEWLINE_S, "\n");
-            back = escape_with (back, ESCAPE_COMMA_S, ",");
-
-            c = card_new ();
-
-            card_set_front (c, front);
-            card_set_back (c, back);
-
-            g_free (front);
-            g_free (back);
-
-            file_add_loaded_card (f, c);
-        }
-        g_strfreev (pair);
-    }
-
-    g_strfreev (lines);
     g_free (contents);
 
-    if (file_get_n_cards (f) == 0) {
+
+    /* Now parse the file. */
+
+    if ((csv = csv_open_r (fname, delimiter, err)) == NULL)
+        return NULL;
+
+    f = file_new ();
+
+    while ((row = csv_read_row (csv, &parse_err)) != NULL)
+        load_card_from_csv_row (f, row);
+
+    csv_close (csv);
+
+    if (parse_err != NULL) {
+        g_set_error (err, 0, 0, _("Error while parsing file."));
+        file_free (f, TRUE);
+        f = NULL;
+        g_error_free (parse_err);
+    }
+    else if (file_get_n_cards (f) == 0) {
         g_set_error (err, 0, 0, _("No cards were found in the imported file."));
         file_free (f, TRUE);
         f = NULL;
@@ -603,7 +714,7 @@ fileio_import_sv (const gchar *fname, const gchar *sep, GError **err)
 
 
 File*
-fileio_import (Ignuit *ig, const gchar *fname, const gchar *filter,
+fileio_import_xml (Ignuit *ig, const gchar *fname, const gchar *filter,
     GError **err)
 {
     xsltStylesheetPtr cur = NULL;
@@ -677,49 +788,39 @@ fileio_import (Ignuit *ig, const gchar *fname, const gchar *filter,
 
 
 gboolean
-fileio_export_sv (File *f, const gchar *fname, const gchar *sep, GError **err)
+fileio_export_csv (File *f, const gchar *fname, gchar delimiter, GError **err)
 {
-    gchar *front, *back;
-    FILE  *f_strm;
     GList *cur;
-    Card  *c;
+    Csv *csv;
+    Card *c;
 
 
     /* Export a file as Comma or Tab Separated Values. */
 
-    f_strm = fopen (fname, "w");
-    if (!f_strm) {
-        g_set_error (err, 0, 0, g_strerror (errno));
+    if ((csv = csv_open_w (fname, delimiter, err)) == NULL)
         return FALSE;
-    }
 
     for (cur = file_get_cards (f); cur != NULL; cur = cur->next) {
 
         c = CARD(cur);
 
-        front = g_strdup (card_get_front (c));
-        front = escape_with (front, "\n", ESCAPE_NEWLINE_S);
-        front = escape_with (front, ",", ESCAPE_COMMA_S);
+        csv_row_clear (csv);
+        csv_row_add_field (csv, card_get_front (c));
+        csv_row_add_field (csv, card_get_back (c));
+        csv_row_add_field (csv, category_get_title (card_get_category (c)));
 
-        back = g_strdup (card_get_back (c));
-        back = escape_with (back, "\n", ESCAPE_NEWLINE_S);
-        back = escape_with (back, ",", ESCAPE_COMMA_S);
-
-        fprintf (f_strm, "%s%s%s\n", front, sep, back);
-
-        g_free (front);
-        g_free (back);
+        csv_write_row (csv);
 
     }
 
-    fclose (f_strm);
+    csv_close (csv);
 
     return TRUE;
 }
 
 
 gboolean
-fileio_export (Ignuit *ig, const gchar *fname, const gchar *filter,
+fileio_export_xml (Ignuit *ig, const gchar *fname, const gchar *filter,
     GError **err)
 {
     xsltStylesheetPtr cur = NULL;
