@@ -27,10 +27,9 @@
 
 void    latex_init (void);
 void    latex_free (void);
-void    latex_set_dpi (gint n);
 
 gchar*  latex_preprocess (const gchar *text, const gchar *wdir,
-            gboolean do_render);
+            gint latex_dpi, gboolean do_render);
 
 
 #endif /* HAVE_LATEX_H */
