@@ -52,19 +52,19 @@ signal_handler (int signum)
         return;
     }
 
-    if (ignuit_for_signal == NULL || ignuit_for_signal->file == NULL || file_get_changed(ignuit_for_signal->file) == FALSE) {
+    if (ignuit_for_signal == NULL || ignuit_for_signal->file == NULL
+        || file_get_changed (ignuit_for_signal->file) == FALSE) {
         exit (1);
     }
 
     filename = file_get_filename (ignuit_for_signal->file);
-    if (filename == NULL) {
+    if (filename == NULL)
         exit (1);
-    }
 
     if (!fileio_save (ignuit_for_signal, filename, &err)) {
         if (err != NULL) {
             g_printerr ("Saving '%s' failed: %s\n", filename, err->message);
-            g_free (err);
+            g_error_free (err);
             exit (1);
         }
         else {
@@ -106,7 +106,9 @@ install_signal_handlers (void)
     sigemptyset (&new_action.sa_mask);
     new_action.sa_flags = 0;
 
-    /* Block the signals we handle during handler execution to avoid race conditions. */
+    /* Block the signals we handle during handler execution to avoid
+     * race conditions. */
+
     sigaddset (&new_action.sa_mask, SIGINT);
     sigaddset (&new_action.sa_mask, SIGTERM);
     sigaddset (&new_action.sa_mask, SIGHUP);
@@ -125,9 +127,9 @@ get_current_hour (void)
 
     /* Return the number of hours past midnight (0..23). */
 
-    if (time (&t) == -1 || (lt = localtime (&t)) == NULL) {
+    if (time (&t) == -1 || (lt = localtime (&t)) == NULL)
         return 0;
-    }
+
     return lt->tm_hour;
 }
 
@@ -140,9 +142,9 @@ get_current_minute (void)
 
     /* Return the number of minutes past the hour (0..59). */
 
-    if (time (&t) == -1 || (lt = localtime (&t)) == NULL) {
+    if (time (&t) == -1 || (lt = localtime (&t)) == NULL)
         return -1;
-    }
+
     return lt->tm_min;
 }
 
@@ -154,10 +156,11 @@ date_today (GDate *date)
 
     time_t t;
 
-    if (time (&t) == -1) {
+    if (time (&t) == -1)
         return FALSE;
-    }
+
     g_date_set_time_t (date, t);
+
     return TRUE;
 }
 
@@ -166,8 +169,7 @@ gchar*
 date_str (gchar *dest, GDate *date)
 {
     /* Fill dest with the date in YYYY-MM-DD format, '\0' terminated.
-     * The dest string must be already allocated to at least 11 chars.
-     */
+     * The dest string must be already allocated to at least 11 chars. */
 
     if (date == NULL) {
         dest[0] = '-';
@@ -187,7 +189,6 @@ date_str (gchar *dest, GDate *date)
 void
 ignuit_free (Ignuit *ig)
 {
-
     file_free (ig->file, TRUE);
     prefs_free (ig->prefs);
 
@@ -221,9 +222,8 @@ ig_clear_clipboard (Ignuit *ig)
 void
 ig_add_clipboard (Ignuit *ig, Card *c)
 {
-    if (ig->clipboard == NULL) {
+    if (ig->clipboard == NULL)
         ig->clipboard = category_new ("");
-    }
 
     category_prepend_card (ig->clipboard, c);
 }
@@ -243,6 +243,18 @@ ig_category_is_clipboard (Ignuit *ig, Category *cat)
 }
 
 
+void
+ig_file_changed (Ignuit *ig)
+{
+    /* Let the user know that there are unsaved changes. */
+
+    if (file_get_changed (ig->file) == FALSE) {
+        file_set_changed (ig->file, TRUE);
+        app_window_update_title (ig);
+    }
+}
+
+
 int
 main (int argc, char *argv[])
 {
@@ -258,9 +270,9 @@ main (int argc, char *argv[])
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
 
-    if (!g_thread_supported ()) {
+    if (!g_thread_supported ())
         g_thread_init (NULL);
-    }
+
     context = g_option_context_new (PACKAGE);
     gstreamer_group = gst_init_get_option_group ();
     g_option_context_add_group (context, gstreamer_group);
@@ -307,7 +319,6 @@ main (int argc, char *argv[])
     }
 
     latex_init ();
-    latex_set_dpi (prefs_get_latex_dpi (ig->prefs));
 
     gst_init (&argc, &argv);
 
