@@ -34,6 +34,7 @@
 
 #define F_GLADE_MAIN            GLADE_DIR G_DIR_SEPARATOR_S "main.glade"
 #define F_GLADE_EDITOR          GLADE_DIR G_DIR_SEPARATOR_S "editor.glade"
+#define F_GLADE_TAGGER          GLADE_DIR G_DIR_SEPARATOR_S "tagger.glade"
 #define F_GLADE_QUIZ            GLADE_DIR G_DIR_SEPARATOR_S "quiz.glade"
 #define F_GLADE_PROPERTIES      GLADE_DIR G_DIR_SEPARATOR_S "properties.glade"
 #define F_GLADE_PREFERENCES     GLADE_DIR G_DIR_SEPARATOR_S "preferences.glade"
@@ -46,7 +47,7 @@
 #define F_IMG_BTN_QUIZ_1        PACKAGE G_DIR_SEPARATOR_S \
                                 "box-start-quiz-24.png"
 
-#define UNICHAR_TAGGED    "\342\232\221"  /* UTF-8 Black Flag */
+#define UNICHAR_FLAGGED   "\342\232\221"  /* UTF-8 Black Flag */
 #define UNICHAR_EXPIRED   "\342\230\205"  /* UTF-8 Black Star */
 
 
@@ -72,7 +73,7 @@ typedef enum {
     QUIZ_NEW_CARDS,
     QUIZ_EXPIRED_CARDS,
     QUIZ_NEW_AND_EXPIRED_CARDS,
-    QUIZ_TAGGED_CARDS,
+    QUIZ_FLAGGED_CARDS,
     QUIZ_SELECTED_CARDS,
     QUIZ_N_CARD_SELECTIONS
 
@@ -132,16 +133,19 @@ struct _Ignuit {
     GtkWidget    *t_save;
     GtkWidget    *m_start_quiz;
     GtkWidget    *t_start_quiz;
+    GtkWidget    *m_find_flagged;
     GtkWidget    *m_find_all;
     GtkWidget    *m_view_trash;
-    GtkWidget    *m_tag;
+    GtkWidget    *m_edit_tags;
+    GtkWidget    *m_flag;
     GtkWidget    *m_switch_sides;
     GtkWidget    *m_reset_stats;
     GtkWidget    *m_paste_card;
     GtkWidget    *m_select_all;
 
     GtkWidget    *m_card_popup_select_all;
-    GtkWidget    *m_card_popup_tag;
+    GtkWidget    *m_card_popup_edit_tags;
+    GtkWidget    *m_card_popup_flag;
     GtkWidget    *m_card_popup_switch_sides;
     GtkWidget    *m_card_popup_reset_stats;
     GtkWidget    *m_card_popup_paste;
@@ -166,6 +170,7 @@ void        ig_clear_clipboard (Ignuit *ig);
 void        ig_add_clipboard (Ignuit *ig, Card *c);
 Category*   ig_get_clipboard (Ignuit *ig);
 gboolean    ig_category_is_clipboard (Ignuit *ig, Category *cat);
+void        ig_file_changed (Ignuit *ig);
 
 
 #endif /* HAVE_MAIN_H */
