@@ -40,6 +40,8 @@ struct _Prefs {
     gint        app_height;
     gint        quiz_width;
     gint        quiz_height;
+    gint        tagger_width;
+    gint        tagger_height;
     gint        category_pane_width;
     gint        category_column_title_width;
     gint        card_column_front_width;
@@ -47,8 +49,11 @@ struct _Prefs {
     gint        card_column_category_width;
     gint        latex_dpi;
     AutoListen  listen;
+    gboolean    backup;
     gboolean    card_column_visible[CARDS_N_COLUMNS];
     gboolean    sticky_flips;
+    gboolean    quiz_answer_bar_visible;
+    gboolean    editor_tag_bar_visible;
     gboolean    main_toolbar_visible;
     gboolean    category_pane_visible;
     gboolean    bottom_toolbar_visible;
@@ -73,7 +78,7 @@ static gchar *card_column_name[] = {
     "expired",
     "expiry",
     "time",
-    "tagged"
+    "flagged"
 };
 
 
@@ -102,11 +107,14 @@ prefs_csvstr_to_schedules (Prefs *p, const gchar *csvstr)
     /* Set schedules from a string of comma separated values. */
 
     v = g_strsplit (csvstr, ",", N_GROUPS);
+
     for (i = 0; i < N_GROUPS; i++) {
-        if (v[i] == NULL) {
+
+        if (v[i] == NULL)
             break;
-        }
+
         prefs_set_schedule (p, i, atoi (v[i]));
+
     }
     g_strfreev (v);
 }
@@ -121,16 +129,15 @@ prefs_schedules_to_csvstr (Prefs *p)
 
     /* Return a newly allocated string of comma separated schedules. */
 
-    for (i = 0; i < N_GROUPS; i++) {
+    for (i = 0; i < N_GROUPS; i++)
         v[i] = g_strdup_printf ("%d", p->schedule[i]);
-    }
+
     v[N_GROUPS] = NULL;
 
     s = g_strjoinv (",", v);
 
-    for (i = 0; i < N_GROUPS; i++) {
+    for (i = 0; i < N_GROUPS; i++)
         g_free (v[i]);
-    }
 
     return s;
 }
@@ -193,9 +200,8 @@ prefs_colors_to_csvstr (Prefs *p)
 
     s = g_strjoinv (",", v);
 
-    for (i = 0; i < N_CARD_COLORS; i++) {
+    for (i = 0; i < N_CARD_COLORS; i++)
         g_free (v[i]);
-    }
 
     return s;
 }
@@ -214,12 +220,15 @@ prefs_card_columns_to_csvstr (Prefs *p)
     gstr = g_string_new ("");
 
     for (col = 1; col < CARDS_N_COLUMNS; col++) {
+
         if (p->card_column_visible[col]) {
-            if (gstr->len) {
+
+            if (gstr->len)
                 g_string_append_c (gstr, ',');
-            }
+
             g_string_append (gstr, card_column_name[col]);
         }
+
     }
 
     s = gstr->str;
@@ -239,19 +248,18 @@ prefs_csvstr_to_card_columns (Prefs *p, const gchar *csvstr)
 
     /* Set card column visibility from a string of comma separated values. */
 
-    for (col = 1; col < CARDS_N_COLUMNS; col++) {
+    for (col = 1; col < CARDS_N_COLUMNS; col++)
         p->card_column_visible[col] = FALSE;
-    }
 
     v = g_strsplit (csvstr, ",", CARDS_N_COLUMNS);
     i = 0;
 
     while (v[i] != NULL) {
-        for (col = 1; col < CARDS_N_COLUMNS; col++) {
-            if (strcmp (v[i], card_column_name[col]) == 0) {
+
+        for (col = 1; col < CARDS_N_COLUMNS; col++)
+            if (strcmp (v[i], card_column_name[col]) == 0)
                 p->card_column_visible[col] = TRUE;
-            }
-        }
+
         i++;
     }
 
@@ -267,12 +275,29 @@ prefs_csvstr_to_card_columns (Prefs *p, const gchar *csvstr)
         col++;
     }
 
-    if (!ok) {
+    if (!ok)
         p->card_column_visible[COLUMN_CARD_FRONT] = TRUE;
+}
+
+
+void
+prefs_set_backup (Prefs *p, gboolean backup)
+{
+    if (backup != p->backup) {
+        p->backup = backup;
+        gconf_client_set_bool (p->gconf_client,
+            PREF_BACKUP, backup, NULL);
     }
 }
 
 
+gboolean
+prefs_get_backup (Prefs *p)
+{
+    return p->backup;
+}
+
+
 void
 prefs_set_sticky_flips (Prefs *p, gboolean sticky)
 {
@@ -311,7 +336,8 @@ prefs_get_auto_listen (Prefs *p)
 void
 prefs_set_category_pane_width (Prefs *p, gint width)
 {
-    if (width <= 0) { width = DEFAULT_CATEGORY_PANE_WIDTH; }
+    if (width <= 0)
+        width = DEFAULT_CATEGORY_PANE_WIDTH;
 
     if (width != p->category_pane_width) {
         p->category_pane_width = width;
@@ -400,6 +426,42 @@ prefs_get_statusbar_visible (Prefs *p)
 }
 
 
+void
+prefs_set_quiz_answer_bar_visible (Prefs *p, gboolean visible)
+{
+    if (visible != p->quiz_answer_bar_visible) {
+        p->quiz_answer_bar_visible = visible;
+        gconf_client_set_bool (p->gconf_client,
+            PREF_QUIZ_ANSWERBAR_VISIBLE, visible, NULL);
+    }
+}
+
+
+gboolean
+prefs_get_quiz_answer_bar_visible (Prefs *p)
+{
+    return p->quiz_answer_bar_visible;
+}
+
+
+void
+prefs_set_editor_tag_bar_visible (Prefs *p, gboolean visible)
+{
+    if (visible != p->editor_tag_bar_visible) {
+        p->editor_tag_bar_visible = visible;
+        gconf_client_set_bool (p->gconf_client,
+            PREF_EDITOR_TAGBAR_VISIBLE, visible, NULL);
+    }
+}
+
+
+gboolean
+prefs_get_editor_tag_bar_visible (Prefs *p)
+{
+    return p->editor_tag_bar_visible;
+}
+
+
 void
 prefs_set_color_gdk (Prefs *p, Color which, GdkColor *color)
 {
@@ -428,9 +490,8 @@ prefs_set_color (Prefs *p, Color which, const gchar *spec)
 {
     GdkColor color;
 
-    if (gdk_color_parse (spec, &color)) {
+    if (gdk_color_parse (spec, &color))
         prefs_set_color_gdk (p, which, &color);
-    }
 }
 
 
@@ -467,14 +528,18 @@ prefs_set_schedule (Prefs *p, Group g, gint days)
 void
 prefs_set_app_size (Prefs *p, gint width, gint height)
 {
-    if (width < 1) { width = DEFAULT_APP_WIDTH; }
-    if (height < 1) { height = DEFAULT_APP_HEIGHT; }
+    if (width < 1)
+        width = DEFAULT_APP_WIDTH;
+
+    if (height < 1)
+        height = DEFAULT_APP_HEIGHT;
 
     if (width != p->app_width) {
         p->app_width = width;
         gconf_client_set_int (p->gconf_client,
             PREF_APP_WIDTH, width, NULL);
     }
+
     if (height != p->app_height) {
         p->app_height = height;
         gconf_client_set_int (p->gconf_client,
@@ -500,14 +565,18 @@ prefs_get_app_height (Prefs *p)
 void
 prefs_set_quiz_size (Prefs *p, gint width, gint height)
 {
-    if (width < 1) { width = DEFAULT_QUIZ_WIDTH; }
-    if (height < 1) { height = DEFAULT_QUIZ_HEIGHT; }
+    if (width < 1)
+        width = DEFAULT_QUIZ_WIDTH;
+
+    if (height < 1)
+        height = DEFAULT_QUIZ_HEIGHT;
 
     if (width != p->quiz_width) {
         p->quiz_width = width;
         gconf_client_set_int (p->gconf_client,
             PREF_QUIZ_WIDTH, width, NULL);
     }
+
     if (height != p->quiz_height) {
         p->quiz_height = height;
         gconf_client_set_int (p->gconf_client,
@@ -530,10 +599,48 @@ prefs_get_quiz_height (Prefs *p)
 }
 
 
+void
+prefs_set_tagger_size (Prefs *p, gint width, gint height)
+{
+    if (height < 1)
+        height = DEFAULT_TAGGER_HEIGHT;
+
+    if (width < 1)
+        width = DEFAULT_TAGGER_WIDTH;
+
+    if (width != p->tagger_width) {
+        p->tagger_width = width;
+        gconf_client_set_int (p->gconf_client,
+            PREF_TAGGER_WIDTH, width, NULL);
+    }
+
+    if (height != p->tagger_height) {
+        p->tagger_height = height;
+        gconf_client_set_int (p->gconf_client,
+            PREF_TAGGER_HEIGHT, height, NULL);
+    }
+}
+
+
+gint
+prefs_get_tagger_width (Prefs *p)
+{
+    return p->tagger_width;
+}
+
+
+gint
+prefs_get_tagger_height (Prefs *p)
+{
+    return p->tagger_height;
+}
+
+
 void
 prefs_set_category_column_title_width (Prefs *p, gint width)
 {
-    if (width <= 0) { width = DEFAULT_CATEGORY_COLUMN_TITLE_WIDTH; }
+    if (width <= 0)
+        width = DEFAULT_CATEGORY_COLUMN_TITLE_WIDTH;
 
     if (width != p->category_column_title_width) {
         p->category_column_title_width = width;
@@ -553,7 +660,8 @@ prefs_get_category_column_title_width (Prefs *p)
 void
 prefs_set_card_column_front_width (Prefs *p, gint width)
 {
-    if (width <= 0) { width = DEFAULT_CARD_COLUMN_FRONT_WIDTH; }
+    if (width <= 0)
+        width = DEFAULT_CARD_COLUMN_FRONT_WIDTH;
 
     if (width != p->card_column_front_width) {
         p->card_column_front_width = width;
@@ -573,7 +681,8 @@ prefs_get_card_column_front_width (Prefs *p)
 void
 prefs_set_card_column_back_width (Prefs *p, gint width)
 {
-    if (width <= 0) { width = DEFAULT_CARD_COLUMN_BACK_WIDTH; }
+    if (width <= 0)
+        width = DEFAULT_CARD_COLUMN_BACK_WIDTH;
 
     if (width != p->card_column_back_width) {
         p->card_column_back_width = width;
@@ -593,7 +702,8 @@ prefs_get_card_column_back_width (Prefs *p)
 void
 prefs_set_card_column_category_width (Prefs *p, gint width)
 {
-    if (width <= 0) { width = DEFAULT_CARD_COLUMN_CATEGORY_WIDTH; }
+    if (width <= 0)
+        width = DEFAULT_CARD_COLUMN_CATEGORY_WIDTH;
 
     if (width != p->card_column_category_width) {
         p->card_column_category_width = width;
@@ -638,8 +748,8 @@ prefs_get_card_column_visible (Prefs *p, gint col)
 void
 prefs_set_card_font (Prefs *p, const gchar *fontname)
 {
-    if (p->card_font == NULL ||
-        (g_ascii_strcasecmp (p->card_font, fontname) != 0)) {
+    if (p->card_font == NULL
+        || (g_ascii_strcasecmp (p->card_font, fontname) != 0)) {
 
             g_free (p->card_font);
             p->card_font = g_strdup (fontname);
@@ -730,7 +840,8 @@ prefs_get_confirm_empty_trash (Prefs *p)
 void
 prefs_set_latex_dpi (Prefs *p, gint dpi)
 {
-    if (dpi < 0 || dpi > 500) { dpi = DEFAULT_LATEX_DPI; }
+    if (dpi < 0 || dpi > MAX_LATEX_DPI)
+        dpi = DEFAULT_LATEX_DPI;
 
     if (dpi != p->latex_dpi) {
         p->latex_dpi = dpi;
@@ -790,17 +901,30 @@ prefs_load (void)
 
     b = gconf_client_get_bool (p->gconf_client, PREF_MAIN_TOOLBAR_VISIBLE, NULL);
     prefs_set_main_toolbar_visible (p, b);
+
     b = gconf_client_get_bool (p->gconf_client, PREF_CATEGORY_PANE_VISIBLE, NULL);
     prefs_set_category_pane_visible (p, b);
+
     b = gconf_client_get_bool (p->gconf_client, PREF_BOTTOM_TOOLBAR_VISIBLE, NULL);
     prefs_set_bottom_toolbar_visible (p, b);
+
     b = gconf_client_get_bool (p->gconf_client, PREF_STATUSBAR_VISIBLE, NULL);
     prefs_set_statusbar_visible (p, b);
 
+    b = gconf_client_get_bool (p->gconf_client, PREF_QUIZ_ANSWERBAR_VISIBLE, NULL);
+    prefs_set_quiz_answer_bar_visible (p, b);
+
+    b = gconf_client_get_bool (p->gconf_client, PREF_EDITOR_TAGBAR_VISIBLE, NULL);
+    prefs_set_editor_tag_bar_visible (p, b);
+
     w = gconf_client_get_int (p->gconf_client, PREF_QUIZ_WIDTH, NULL);
     h = gconf_client_get_int (p->gconf_client, PREF_QUIZ_HEIGHT, NULL);
     prefs_set_quiz_size (p, w, h);
 
+    w = gconf_client_get_int (p->gconf_client, PREF_TAGGER_WIDTH, NULL);
+    h = gconf_client_get_int (p->gconf_client, PREF_TAGGER_HEIGHT, NULL);
+    prefs_set_tagger_size (p, w, h);
+
     s = gconf_client_get_string (p->gconf_client, PREF_CARD_FONT, NULL);
     prefs_set_card_font (p, s ? s : DEFAULT_CARD_FONT);
     g_free (s);
@@ -813,13 +937,15 @@ prefs_load (void)
     prefs_csvstr_to_schedules (p, s ? s : DEFAULT_SCHEDULES);
     g_free (s);
 
-    if ((s = gconf_client_get_string (p->gconf_client, PREF_WORKDIR, NULL))) {
+    if ((s = gconf_client_get_string (p->gconf_client, PREF_WORKDIR, NULL)))
         prefs_set_workdir (p, s);
-    }
 
     b = gconf_client_get_bool (p->gconf_client, PREF_FIND_WITH_REGEX, NULL);
     prefs_set_find_with_regex (p, b);
 
+    b = gconf_client_get_bool (p->gconf_client, PREF_BACKUP, NULL);
+    prefs_set_backup (p, b);
+
     p->confirm_empty_trash = TRUE;
     b = gconf_client_get_bool (p->gconf_client, PREF_CONFIRM_EMPTY_TRASH, NULL);
     prefs_set_confirm_empty_trash (p, b);
@@ -839,11 +965,9 @@ prefs_free (Prefs *p)
     g_free (p->card_font);
     g_free (p->workdir);
 
-    for (i = 0; i < N_CARD_COLORS; i++) {
-        if (p->color[i]) {
+    for (i = 0; i < N_CARD_COLORS; i++)
+        if (p->color[i])
             gdk_color_free (p->color[i]);
-        }
-    }
 
     g_free (p);
 }
