@@ -39,7 +39,7 @@ typedef enum {
     TAG_OPEN = 0,
     TAG_CLOSE,
     TAG_SELF_CLOSE
-} TagType;
+} MTagType;
 
 
 typedef enum {
@@ -134,7 +134,7 @@ struct _Parser {
     Token     token;
     gint      length;
     ElementID element_id;
-    TagType   tag_type; /* open, close, or self-closing */
+    MTagType  tag_type; /* open, close, or self-closing */
     gboolean  in_keyword;
     gboolean  verbatim;
     gboolean  error;
@@ -145,9 +145,9 @@ struct _Parser {
 };
 
 
-typedef struct _Tag Tag;
+typedef struct _MTag MTag;
 
-struct _Tag {
+struct _MTag {
 
     ElementID element_id;
     GList     *attr;
@@ -375,10 +375,8 @@ attr_list_get_value (GList *attr, const gchar *name)
 
 
 static void
-tag_free (Tag *t)
+mtag_free (MTag *t)
 {
-    g_assert (t != NULL);
-
     if (t->attr != NULL) {
 
         GList *cur;
@@ -417,7 +415,7 @@ parser_free (Parser *p)
         GList *cur;
 
         for (cur = p->tags; cur != NULL; cur = cur->next) {
-            tag_free ((Tag*)cur->data);
+            mtag_free ((MTag*)cur->data);
         }
         g_list_free (p->tags);
 
@@ -508,7 +506,7 @@ parse_attr (const gchar *str, gint *len)
 
 
 static GList*
-parse_tag (const gchar *str, TagType *tag_type, gint *length, ElementID *id)
+parse_tag (const gchar *str, MTagType *tag_type, gint *length, ElementID *id)
 {
     GList *attr;
     Attr *a;
@@ -682,11 +680,11 @@ static void
 display_open_tag (GtkTextBuffer *textbuf, Parser *p)
 {
     GtkTextIter start;
-    Tag *tag;
+    MTag *tag;
 
     gtk_text_buffer_get_end_iter (textbuf, &start);
 
-    tag = g_new0 (Tag, 1);
+    tag = g_new0 (MTag, 1);
     tag->element_id = p->element_id;
     tag->offset = gtk_text_iter_get_offset (&start);
     tag->attr = p->attr;
@@ -700,7 +698,7 @@ static void
 display_close_tag (GtkTextView *textview, GtkTextBuffer *textbuf, Parser *p)
 {
     GtkTextIter start, end;
-    Tag *tag;
+    MTag *tag;
 
 
     if (p->tags == NULL) {
@@ -724,7 +722,7 @@ display_close_tag (GtkTextView *textview, GtkTextBuffer *textbuf, Parser *p)
 
         if (tag->element_id < ELEMENT_ID_BR) {
 
-            /* if not a self-closing tag */
+            /* If not a self-closing tag... */
 
             const gchar *s;
 
@@ -764,7 +762,7 @@ display_close_tag (GtkTextView *textview, GtkTextBuffer *textbuf, Parser *p)
 
         p->tags = g_list_delete_link (p->tags, g_list_first (p->tags));
 
-        tag_free (tag);
+        mtag_free (tag);
     }
 }
 
@@ -854,7 +852,8 @@ textview_display_with_markup (Ignuit *ig, GtkTextView *textview,
         ttext = g_strdup (text);
     }
     else {
-        ttext = latex_preprocess (text, workdir, TRUE);
+        gint dpi = prefs_get_latex_dpi (ig->prefs);
+        ttext = latex_preprocess (text, workdir, dpi, TRUE);
 #if 0
         g_print ("%s\n", ttext);
 #endif
@@ -930,7 +929,8 @@ remove_card_markup (const gchar *text)
 
     /* Return a newly allocated string with markup removed. */
 
-    if (text == NULL) { text = ""; }
+    if (text == NULL)
+        text = "";
 
     ttext = latex_preprocess (text, NULL, FALSE);
 #if 0
